import dataclasses

from obspy.io.reftek.util import bcd

from sohstationviewer.model.mseed_data.record_reader_helper import Unpacker
from sohstationviewer.model.reftek.rt130_experiment.header import PacketHeader

import numpy


@dataclasses.dataclass
class SOHExtendedHeader:
    event_number: int
    data_stream_number: int
    channel_number: int
    number_of_samples: int
    flags: int
    data_format: str


@dataclasses.dataclass
class SOHPacket:
    header: PacketHeader
    extended_header: SOHExtendedHeader
    data: bytes


def bcd_16bit_int(_i):
    _i = bcd(_i)
    return _i[0] * 100 + _i[1]


def read_soh_packet(packet: bytes, unpacker: Unpacker):
    event_number = bcd_16bit_int(numpy.frombuffer(packet[16:18], numpy.uint8))
    data_stream_number = bcd(numpy.frombuffer(packet[18:19], numpy.uint8))
    channel_number = bcd(numpy.frombuffer(packet[19:20], numpy.uint8))
    number_of_samples = bcd_16bit_int(numpy.frombuffer(packet[20:22], numpy.uint8))
    flags = unpacker.unpack('B', packet[22:23])[0]
    data_format = packet[23:24].hex().upper()

    extended_header = SOHExtendedHeader(event_number, data_stream_number,
                                        channel_number, number_of_samples,
                                        flags, data_format)
    payload = packet[24:]
    return extended_header, payload
